/**
 *******************************************************************************
 * @file    tmpm366_dmac_int.c
 * @brief   test M366 SPI Host to Slave (SSP0 to SSP1)
 *           for the TOSHIBA 'TMPM366' Device Series
 * @version V0.101
 * @date    2014/05/20
 *
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 *
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "tmpm366_ssp.h"
#include "tmpm366_gpio.h"
#include "dma.h"

typedef enum { SAME = 1U, NOT_SAME = !SAME } TestResult;

void DisplayLED(uint16_t dat);
void initSSP(void);
void GPIO_SetSSP(void);
TestResult Buffercompare(const uint16_t * Src, const uint16_t * Dst, uint32_t Len);

#define MAX_BUFSIZE  64U

TestResult result = NOT_SAME;

/* to check the effect due to too low speed, uncomment the line below */
/* #define BITRATE_MIN */

/*------------------------------------------------------------------------
	SSP0 pins	            Note	                SSP1 pins
1	PD7/SP0FSS  J1.pin16)	Connect PD7 to PB3	    PB3/SP1FSS  (J1.pin20)
2	PD6/SP0CLK (J1.pin15)	Connect PD6 to PB2	    PB2/SP1CLK  (J1.pin21)
3	PD5/SP0DI  (J1.pin14)	Connect PD5 to PB0	    PB1/SP1DI   (J1.pin22)
4	PD4/SP0DO  (J1.pin13)	Connect PD4 to PB1	    PB0/SP1DO   (J1.pin23)

	Connect J1:   pin 16 to pin 20,     pin 15 to pin 21,     pin 14 to pin 23,     pin 13 to pin 22

CR.x: call GPIO_SetOutputEnableReg()
PD0F2: call: GPIO_EnableFuncReg()
IE.x: call: GPIO_SetInputEnableReg()
OD.x: call: GPIO_SetOpenDrain()

to use GPIO as SSP0 (SPI Host),
//========
PD4 to be used as SP0DO:
 set: PDCR.4,  GPIO_SetOutputEnableReg(GPIO_PD, GPIO_BIT_4, ENABLE);
 set: PD4F2,   GPIO_EnableFuncReg(GPIO_PD, GPIO_FUNC_REG_2, GPIO_BIT_4);
 clear: PDIE.4, GPIO_SetInputEnableReg(GPIO_PD, GPIO_BIT_4, DISABLE);

PD5 to be used as SP0DI:
 set: PDIE.5,   GPIO_SetInputEnableReg(GPIO_PD, GPIO_BIT_5, ENABLE);
 set: PD5F2    GPIO_EnableFuncReg(GPIO_PD, GPIO_FUNC_REG_2, GPIO_BIT_5);
 clear: PDCR.5, GPIO_SetOutputEnableReg(GPIO_PD, GPIO_BIT_5, DISABLE);
 clear: PDOD.5  GPIO_SetOpenDrain(GPIO_PD, GPIO_BIT_5, DISABLE);

PD6 to be used as SP0CLK out:
 set: PDCR.6,   GPIO_SetOutputEnableReg(GPIO_PD, GPIO_BIT_6, ENABLE);
 set: PD6F2     GPIO_EnableFuncReg(GPIO_PD, GPIO_FUNC_REG_2, GPIO_BIT_6);
 clear: PDIE.6  GPIO_SetInputEnableReg(GPIO_PD, GPIO_BIT_6, DISABLE);

PD7 to be used as SP0FSS out:
 set: PDCR.7,   GPIO_SetOutputEnableReg(GPIO_PD, GPIO_BIT_7, ENABLE);
 set: PD7F2     GPIO_EnableFuncReg(GPIO_PD, GPIO_FUNC_REG_2, GPIO_BIT_7);
 clear: PDIE.7  GPIO_SetInputEnableReg(GPIO_PD, GPIO_BIT_7, DISABLE);
//========

to use GPIO as SSP1 (SPI Slave),
//=========================
PB0 to be used as SP1DO:
 set: PBCR.0,   GPIO_SetOutputEnableReg(GPIO_PB, GPIO_BIT_0, ENABLE);
 set: PB0F2,    GPIO_EnableFuncReg(GPIO_PB, GPIO_FUNC_REG_2, GPIO_BIT_0);
 clear: PBIE.0  GPIO_SetInputEnableReg(GPIO_PB, GPIO_BIT_0, DISABLE);

PB1 to be used as SP1DI:
 set: PBIE.1,   GPIO_SetInputEnableReg(GPIO_PB, GPIO_BIT_1, ENABLE);
 set: PB1F2     GPIO_EnableFuncReg(GPIO_PB, GPIO_FUNC_REG_2, GPIO_BIT_1);
 clear: PBCR.1, GPIO_SetOutputEnableReg(GPIO_PB, GPIO_BIT_1, DISABLE);
 clear: PBOD.1  GPIO_SetOpenDrain(GPIO_PB, GPIO_BIT_1, DISABLE);

PB2 to be used as SP1CLK in:
 set: PBIE.2,   GPIO_SetInputEnableReg(GPIO_PB, GPIO_BIT_2, ENABLE);
 set: PB2F2     GPIO_EnableFuncReg(GPIO_PB, GPIO_FUNC_REG_2, GPIO_BIT_2);
 clear: PBCR.2, GPIO_SetOutputEnableReg(GPIO_PB, GPIO_BIT_2, DISABLE);
 clear: PBOD.2, GPIO_SetOpenDrain(GPIO_PB, GPIO_BIT_2, DISABLE);

PB3 to be used as SP1FSS in:
 set: PBIE.3,   GPIO_SetInputEnableReg(GPIO_PB, GPIO_BIT_3, ENABLE);
 set: PB3F2     GPIO_EnableFuncReg(GPIO_PB, GPIO_FUNC_REG_2, GPIO_BIT_3);
 clear: PBCR.3, GPIO_SetOutputEnableReg(GPIO_PB, GPIO_BIT_3, DISABLE);
 clear: PBOD.3, GPIO_SetOpenDrain(GPIO_PN, GPIO_BIT_3, DISABLE);

------------------------------------------------------------------------*/
void GPIO_SetSSP(void)
{
    /* PD4 to be used as SP0DO: */
    GPIO_SetOutputEnableReg(GPIO_PD, GPIO_BIT_4, ENABLE);
    GPIO_EnableFuncReg(GPIO_PD, GPIO_FUNC_REG_2, GPIO_BIT_4);
    GPIO_SetInputEnableReg(GPIO_PD, GPIO_BIT_4, DISABLE);

    /* PD5 to be used as SP0DI: */
    GPIO_SetInputEnableReg(GPIO_PD, GPIO_BIT_5, ENABLE);
    GPIO_EnableFuncReg(GPIO_PD, GPIO_FUNC_REG_2, GPIO_BIT_5);
    GPIO_SetOutputEnableReg(GPIO_PD, GPIO_BIT_5, DISABLE);
    GPIO_SetOpenDrain(GPIO_PD, GPIO_BIT_5, DISABLE);

    /* PD6 to be used as SP0CLK out: */
    GPIO_SetOutputEnableReg(GPIO_PD, GPIO_BIT_6, ENABLE);
    GPIO_EnableFuncReg(GPIO_PD, GPIO_FUNC_REG_2, GPIO_BIT_6);
    GPIO_SetInputEnableReg(GPIO_PD, GPIO_BIT_6, DISABLE);

    /* PD7 to be used as SP0FSS out: */
    GPIO_SetOutputEnableReg(GPIO_PD, GPIO_BIT_7, ENABLE);
    GPIO_EnableFuncReg(GPIO_PD, GPIO_FUNC_REG_2, GPIO_BIT_7);
    GPIO_SetInputEnableReg(GPIO_PD, GPIO_BIT_7, DISABLE);


    /* PB0 to be used as SP1DO: */
    GPIO_SetOutputEnableReg(GPIO_PB, GPIO_BIT_0, ENABLE);
    GPIO_EnableFuncReg(GPIO_PB, GPIO_FUNC_REG_2, GPIO_BIT_0);
    GPIO_SetInputEnableReg(GPIO_PB, GPIO_BIT_0, DISABLE);

    /* PB1 to be used as SP1DI: */
    GPIO_SetInputEnableReg(GPIO_PB, GPIO_BIT_1, ENABLE);
    GPIO_EnableFuncReg(GPIO_PB, GPIO_FUNC_REG_2, GPIO_BIT_1);
    GPIO_SetOutputEnableReg(GPIO_PB, GPIO_BIT_1, DISABLE);
    GPIO_SetOpenDrain(GPIO_PB, GPIO_BIT_1, DISABLE);

    /* PB2 to be used as SP1CLK in: */
    GPIO_SetInputEnableReg(GPIO_PB, GPIO_BIT_2, ENABLE);
    GPIO_EnableFuncReg(GPIO_PB, GPIO_FUNC_REG_2, GPIO_BIT_2);
    GPIO_SetOutputEnableReg(GPIO_PB, GPIO_BIT_2, DISABLE);
    GPIO_SetOpenDrain(GPIO_PB, GPIO_BIT_2, DISABLE);

    /* PB3 to be used as SP1FSS in: */
    GPIO_SetInputEnableReg(GPIO_PB, GPIO_BIT_3, ENABLE);
    GPIO_EnableFuncReg(GPIO_PB, GPIO_FUNC_REG_2, GPIO_BIT_3);
    GPIO_SetOutputEnableReg(GPIO_PB, GPIO_BIT_3, DISABLE);
    GPIO_SetOpenDrain(GPIO_PB, GPIO_BIT_3, DISABLE);

}

void initSSP(void)
{
    SSP_InitTypeDef initSSP;

    /* init SSP0 as SPI Host and SSP1 as SPI Slave */

    /* configure the SSP module */
    initSSP.FrameFormat = SSP_FORMAT_SPI;

    /* set bit rate */
    initSSP.PreScale = 8U;
    initSSP.ClkRate = 8U;
    /* define BITRATE_MIN to run at minimum bit rate */
    /*   BitRate = fSYS / (PreScale x (1 + ClkRate)) */
#ifdef BITRATE_MIN
    initSSP.PreScale = 254U;
    initSSP.ClkRate = 255U;
#endif
    initSSP.ClkPolarity = SSP_POLARITY_LOW;
    initSSP.ClkPhase = SSP_PHASE_FIRST_EDGE;
    initSSP.DataSize = 16U;
    initSSP.Mode = SSP_MASTER;
    SSP_Init(TSB_SSP0, &initSSP);

    initSSP.ClkPhase = SSP_PHASE_FIRST_EDGE;
    initSSP.Mode = SSP_SLAVE;
    SSP_Init(TSB_SSP1, &initSSP);

    /* enable DMA in SSP module */
    SSP_SetDMACtrl(TSB_SSP0, SSP_TX, ENABLE);
    SSP_SetDMACtrl(TSB_SSP1, SSP_RX, ENABLE);

    /* enable and run SSP module */
    SSP_Enable(TSB_SSP0);
    SSP_Enable(TSB_SSP1);
}

int main(void)
{
    GPIO_SetSSP();
    initSSP();
    InitDMA();

    /* Wait the end of transmission */
    while (TxEndFlag != DONE) {
        /* Do nothing */
    }

    /* now DMA is finished, Set a Break Point here, */
    /* after function Buffercompare() is called, result == SAME */
    result = Buffercompare(SRC_Buffer, DST_Buffer, BUFFER_SIZE);

    while (1) {
        /* do nothing */
    }
}

/**
  * @brief  To compare two half word buffer
  * @param  Src : pointer to point source buffer
  * @param  Dst : pointer to point destination buffer
  * @param  Len : how many data will be compared
  * @retval SAME: two buffer contain same data
            NOT_SAME: two buffer contain different data
  */
TestResult Buffercompare(const uint16_t * Src, const uint16_t * Dst, uint32_t Len)
{
    uint32_t tmp = 0U;
    TestResult result = NOT_SAME;

    for (tmp = 0U; tmp < Len; tmp++) {
        if (Src[tmp] != Dst[tmp]) {
            break;
        } else {
            result = SAME;
        }
    }

    return result;
}

#ifdef DEBUG
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif
