/**
 *******************************************************************************
 * @file    msc_ramdisk.c
 * @brief   MSC(BOT/SCSI) RAM Disk Sample Code for TOSHIBA 'TMPM366' Device Series
 * @version V0.202
 * @date    2014/05/20
 *
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 *
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

#include <string.h>
#include "usbd_hw.h"
#include "usbd_hw_com.h"
#include "usbd_var.h"
#include "usbd_device_request.h"
#include "usbd_typedefs.h"
#include "usbd_descriptor_msc.h"
#include "usbd_msc.h"
#include "usbd_scsi.h"
#include "usbd_trans.h"

/**
  * @brief Block size of RAM Disk
  */
#define BLOCK_SIZE  USBD_MSC_SCSI_BLOCK_SIZE
/**
  * @brief Number of blocks in the RAM Disk
  */
#define BLOCK_NUM   USBD_MSC_SCSI_BLOCK_NUM
/**
  * @brief RAM disk area (Size : 48 * 512 = 24KB)
  */
uint8_t g_aucRamDisk[BLOCK_NUM][BLOCK_SIZE];
/**
  * @brief Callback information for MSC(SCSI)
  */
SCSI_CB_t gSample_MSC_CB;
/**
  * @brief Don't care status
  */
bool gbStatDC;

/**
  * @brief Buffer size
  */
#define BUFFER_SIZE (512U)
/**
  * @brief Number of drive
  */
#define DRIVE_NUM   (1U)
/**
  * @brief Send data buffer
  */
static uint8_t l_aucSendBuff[BUFFER_SIZE];
/**
  * @brief Receive data buffer
  */
static uint8_t l_aucRecvBuff[BUFFER_SIZE];
/**
  * @brief Reset recovery state
  */
static bool l_bRRState;
/**
  * @brief Total data length
  */
static uint32_t l_ulDataTransferLen;
/**
  * @brief LBA (Logical Block Address)
  */
static uint32_t l_ulLBA;
/**
  * @brief Number of block
  */
static uint16_t l_usBlockNum;
/**
  * @brief Data length (Already read/write)
  */
static uint32_t l_ulDataLenCount;
/**
  * @brief Number of blocks (Already read/write)
  */
static uint16_t l_usBlockCount;
/**
  * @brief Media status
  */
static uint8_t l_ucMediaStat;

/* Prototypes */
static void    _Sample_ConnStat(bool);
static bool    _Sample_MassStorageReset(void);
static uint8_t _Sample_GetMaxLun(void);
static void    _Sample_MediaRead(uint32_t, uint32_t, uint16_t);
static void    _Sample_MediaWrite(uint32_t, uint32_t, uint16_t);
static void    _Sample_SendData(uint32_t, uint8_t*);
static void    _Sample_RecvData(uint32_t, uint8_t*);
static void    _Sample_RamDisk_Read(uint32_t);
static void    _Sample_RAM_Disk_Init(void);
static void    _Sample_RAM_Disk_Read(uint8_t*, uint32_t);
static void    _Sample_RAM_Disk_Write(uint8_t*, uint32_t);

/******************************************************************************/

/**
  * @brief  MSC sample main routine
  * @param  None
  * @retval None
  */
int main(void)
{
    /* Initialization of an internal variable */
    l_ucMediaStat       = false;
    l_ulDataTransferLen = 0U;
    l_ulLBA             = 0U;
    l_usBlockNum        = 0U;
    l_ulDataLenCount    = 0U;
    l_usBlockCount      = 0U;
    l_bRRState          = true;
    
    /* Initialize buffers */
    memset(l_aucSendBuff, 0x00U, BUFFER_SIZE);
    memset(l_aucRecvBuff, 0x00U, BUFFER_SIZE);

    /* Set MSC(SCSI) initialize parameters */
    gSample_MSC_CB.pfnConnStat         = _Sample_ConnStat;
    gSample_MSC_CB.pfnMassStorageReset = _Sample_MassStorageReset;
    gSample_MSC_CB.pfnGetMaxLun        = _Sample_GetMaxLun;
    gSample_MSC_CB.pfnMediaRead        = _Sample_MediaRead;
    gSample_MSC_CB.pfnMediaWrite       = _Sample_MediaWrite;
    gSample_MSC_CB.pfnSendData         = _Sample_SendData;
    gSample_MSC_CB.pfnRecvData         = _Sample_RecvData;

    /* Initialize MSC(SCSI) driver */
    USBD_MSC_SCSI_Init(&gSample_MSC_CB);

    /* Initialize RAM Disk */
    _Sample_RAM_Disk_Init();

    /* Initialize driver works */
    usbd_initialize_standard_class_work_data();

    /* Initialize USB controller */
    Config_USB();

    /* Forever loop (Interrupt driven sample) */
    while(true);
}

/**
  * @brief  Callback for connect/disconnect
  * @param  bStat : Connect status.
  *   This parameter can be one of the following values :
  *      true(Connect), false(Disconnect)
  * @retval None
  */
static void _Sample_ConnStat(bool bStat)
{
    if (bStat == true) {
        /* Connect */
        if (l_ucMediaStat != true) {
            /* Media connect */
            l_ucMediaStat = true;
            /* Set media connect status */
            USBD_MSC_SCSI_Media_ConnStat(true);
        }
    } else {
        /* Disconnect */
        if (l_ucMediaStat != false) {
            /* Media disconnect */
            l_ucMediaStat = false;
            /* Set media disconnect status */
            USBD_MSC_SCSI_Media_ConnStat(false);
        }
    }

    return;
}

/**
  * @brief  Callback for MASS_STORAGE_RESET
  * @param  None
  * @retval l_bRRState : Reset recovery state.
  *   l_bRRState is only true in sample.
  */
static bool _Sample_MassStorageReset(void)
{
    /* Return reset recovery state */
    return l_bRRState;
}

/**
  * @brief  Callback for GET_MAX_LUN
  * @param  None
  * @return Max LUN (Max Logical Unit Number).
  */
static uint8_t _Sample_GetMaxLun(void)
{
    /* Rerturn Max LUN (Max Logical Unit Number) */
    return (DRIVE_NUM-1U);
}

/**
  * @brief  Callback for read media
  * @param  ulTransferDataLength : Transfer data length.
  * @param  ulLBA : LBA (Logical Block Address).
  * @param  usBlockNum : Number of blocks.
  * @retval None
  */
static void _Sample_MediaRead(uint32_t ulTransferDataLength, uint32_t ulLBA, uint16_t usBlockNum)
{
    /* Initialize transfer information */
    l_ulDataTransferLen = ulTransferDataLength;
    l_ulLBA             = ulLBA;
    l_usBlockNum        = usBlockNum;
    l_ulDataLenCount    = 0U;
    l_usBlockCount      = 0U;

    /* Read RAM Disk */
    _Sample_RamDisk_Read(ulLBA);
    
    return;
}

/**
  * @brief  Callback for write media
  * @param  ulTransferDataLength : Transfer data length.
  * @param  ulLBA : LBA (Logical Block Address).
  * @param  usBlockNum : Number of blocks.
  * @retval None
  */
static void _Sample_MediaWrite(uint32_t ulTransferDataLength, uint32_t ulLBA, uint16_t usBlockNum)
{
    /* Initialize transfer information */
    l_ulDataTransferLen = ulTransferDataLength;
    l_ulLBA             = ulLBA;
    l_usBlockNum        = usBlockNum;
    l_ulDataLenCount    = 0U;
    l_usBlockCount      = 0U;

    /* Receive write data */
    gbStatDC = USBD_MSC_SCSI_MediaWriteData(BUFFER_SIZE, l_aucRecvBuff, (void *)USBD_MSC_SCSI_WRITE);
    
    return;
}

/**
  * @brief  Callback for Send data (Read)
  * @param  ulSize : Send data size.
  * @param  pucData : Send data buffer pointer.
  * @retval None
  */
static void _Sample_SendData(uint32_t ulSize, uint8_t *pucData)
{
    /* Adding the send data size */
    l_ulDataLenCount += ulSize;

    if (l_ulDataTransferLen == l_ulDataLenCount) {
        /* Notify read result (PASSED) */
        USBD_MSC_SCSI_MediaReadResult(USBD_MSC_SCSI_COMMAND_PASSED, l_ulDataLenCount, USBD_MSC_SCSI_NO_STALL);
    } else if (l_ulDataTransferLen > l_ulDataLenCount) {
        if (l_usBlockNum > l_usBlockCount) {
            /* Read RAM Disk */
            _Sample_RamDisk_Read(l_ulLBA + l_usBlockCount);
        } else {
            /* Set Sense Key : Illegal Request(05h) */
            USBD_MSC_SCSI_SetSenseKey(USBD_MSC_SCSI_SENSE_KEY_ILLEGAL_REQUEST);
            /* Notify read result (FAILED) */
            USBD_MSC_SCSI_MediaReadResult(USBD_MSC_SCSI_COMMAND_FAILED, l_ulDataLenCount, USBD_MSC_SCSI_REQ_STALL);
        }
    } else if (l_ulDataTransferLen < l_ulDataLenCount) {
        /* Notify read result (PHASE_ERROR) */
        USBD_MSC_SCSI_MediaReadResult(USBD_MSC_SCSI_COMMAND_PHASE_ERROR, USBD_MSC_SCSI_NO_DATA, USBD_MSC_SCSI_REQ_STALL);
    }

    return;
}

/**
  * @brief  Callback for received data (Write)
  * @param  ulSize : Received data size.
  * @param  pucData : Received data buffer pointer.
  * @retval None
  */
static void _Sample_RecvData(uint32_t ulSize, uint8_t *pucData)
{
    /* Adding the received data size */
    l_ulDataLenCount += ulSize;
    /* Write data to RAM Disk */
    _Sample_RAM_Disk_Write(pucData, (l_ulLBA + l_usBlockCount));
    /* Increment block count */
    l_usBlockCount++;

    if (l_ulDataTransferLen == l_ulDataLenCount) {
        /* Notify write result (PASSED) */
        USBD_MSC_SCSI_MediaWriteResult(USBD_MSC_SCSI_COMMAND_PASSED, l_ulDataLenCount, USBD_MSC_SCSI_NO_STALL);
    } else if (l_ulDataTransferLen > l_ulDataLenCount) {
        if (l_usBlockNum > l_usBlockCount) {
            /* Receive write data */
            gbStatDC = USBD_MSC_SCSI_MediaWriteData(BUFFER_SIZE, l_aucRecvBuff, (void *)USBD_MSC_SCSI_WRITE);
        } else {
            /* Set Sense Key : Illegal Request(05h) */
            USBD_MSC_SCSI_SetSenseKey(USBD_MSC_SCSI_SENSE_KEY_ILLEGAL_REQUEST);
            /* Notify write result (FAILED) */
            USBD_MSC_SCSI_MediaWriteResult(USBD_MSC_SCSI_COMMAND_FAILED, l_ulDataLenCount, USBD_MSC_SCSI_REQ_STALL);
        }
    } else if (l_ulDataTransferLen < l_ulDataLenCount) {
        /* Notify write result (PHASE_ERROR) */
        USBD_MSC_SCSI_MediaWriteResult(USBD_MSC_SCSI_COMMAND_PHASE_ERROR, USBD_MSC_SCSI_NO_DATA, USBD_MSC_SCSI_REQ_STALL);
    }

    return;
}

/**
  * @brief  Read RAM Disk
  * @param  ulLBA : LBA (Logical Block Address).
  * @retval None
  */
static void _Sample_RamDisk_Read(uint32_t ulLBA)
{
    /* Read data from RAM Disk */
    _Sample_RAM_Disk_Read((uint8_t *)l_aucSendBuff, ulLBA);
    /* Increment block count */
    l_usBlockCount++;

    /* Send read data */
    gbStatDC = USBD_MSC_SCSI_MediaReadData(BUFFER_SIZE, l_aucSendBuff, (void *)USBD_MSC_SCSI_READ);

    return;
}

/**
  * @brief  Initialize RAM Disk
  * @param  None
  * @retval None
  */
static void _Sample_RAM_Disk_Init(void)
{
    /* Clear sector 0 */
    memset(g_aucRamDisk[0], 0x00U, BLOCK_SIZE);
}

/**
  * @brief  Read data from RAM Disk
  * @param  pucBuf : Data buffer pointer.
  * @param  ulLBA : LBA (Logical Block Address).
  * @retval None
  */
static void _Sample_RAM_Disk_Read(uint8_t *pucBuf, uint32_t ulLBA)
{
    if (ulLBA < BLOCK_NUM) {
        /* Copy data from RAM Disk to buffer */
        memcpy(pucBuf, (uint8_t *)g_aucRamDisk[ulLBA], BLOCK_SIZE);
    } else {
        /* Write buffer to all 0 Data */
        memset(pucBuf, 0, BLOCK_SIZE);
    }
}

/**
  * @brief  Write data to RAM Disk
  * @param  pucBuf : Data buffer pointer.
  * @param  ulLBA : LBA (Logical Block Address).
  * @retval None
  */
static void _Sample_RAM_Disk_Write(uint8_t *pucBuf, uint32_t ulLBA)
{
    if (ulLBA < BLOCK_NUM) {
        /* Copy data from buffer to RAM Disk */
        memcpy((uint8_t *)g_aucRamDisk[ulLBA], pucBuf, BLOCK_SIZE);
    } else {
        /* Do nothing */
    }
}
#ifdef DEBUG
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif
/* EOC */
